/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: IdManager.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if HAVE_CONFIG_H
# include "config.h"
#endif

#if defined(WIN32)
#include <windows.h>
#include <process.h>
#include <time.h>
#else
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#endif

#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>

#if !defined(NO_LOGGER)
#include "Logger.h"
#endif
#include "ServerCommon.h"
#include "IdManager.h"
#include "SrvConf.h"

using namespace OpenSOAP;
using namespace std;

//for LOG
static const std::string CLASS_SIG = "IdManager";

IdManager::IdManager()
  : idSequence_(0)
{
  SrvConf srvConf;
#if defined(WIN32)
  socketAddr_ = IDMGR_SOCKET_ADDR;
#else
  socketAddr_ = srvConf.getSocketPath() + IDMGR_SOCKET_ADDR;
#endif

}

IdManager::~IdManager()
{
  
}

void IdManager::run()
{
  //===================================================================
  // ʐM 
  //===================================================================
#if defined(WIN32)
	typedef SOCKADDR SockAddr;
	typedef SOCKADDR_IN SockAddrAf;
	SockAddrAf serverAddr;
	SockAddrAf clientAddr;

	SOCKET sockfd = INVALID_SOCKET;
	SOCKET newsockfd = INVALID_SOCKET;
	WSADATA data;
	if (WSAStartup(0x101,&data)) {
		cerr << "** E **: WSAStartup failed." << endl;
		throw exception();
	}
	int af = AF_INET;
	int addrLen = sizeof(SockAddrAf);
	int clientLen = int();
#else
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_un SockAddrAf;

  int af = AF_UNIX;
  int sockfd = int();
  int newsockfd = int();
  SockAddrAf serverAddr;
  SockAddrAf clientAddr;
#if HAVE_TYPE_OF_ADDRLEN_AS_SOCKLEN_T
  socklen_t addrLen = socklen_t(); //socket_t();
  socklen_t clientLen = socklen_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_SIZE_T
  size_t addrLen = size_t();
  size_t clientLen = size_t();
#endif
#if HAVE_TYPE_OF_ADDRLEN_AS_INT
  int addrLen = int();
  int clientLen = int();
#endif

#endif

  //DataRetainerƂ̒ʐM|[g
  if (0 > (sockfd = socket(af, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: open socket failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }
#ifdef DEBUG
  cerr << CLASS_SIG << "::run: socket open OK!" << endl;
#endif

  memset((char*)&serverAddr, 0, sizeof(serverAddr));

#if defined(WIN32)
	serverAddr.sin_family = af;
	serverAddr.sin_addr.s_addr = INADDR_ANY;
	serverAddr.sin_port = htons(socketAddr_);
#else
  serverAddr.sun_family = af;
  strcpy(serverAddr.sun_path, socketAddr_.c_str());
  addrLen = sizeof(serverAddr.sun_family) + strlen(serverAddr.sun_path);

  unlink(socketAddr_.c_str());

  mode_t newMask = 0x000;
  mode_t curMask = umask(newMask);
#endif

  if (0 > bind(sockfd, (SockAddr*)&serverAddr, addrLen)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: bind failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }


#if !defined(WIN32)
  umask(curMask);
#endif

#ifdef DEBUG
  cerr << CLASS_SIG << "::run: bind OK!" << endl;
#endif

  if (0 > listen(sockfd, 5)) {
    cerr << TAG_ERR << CLASS_SIG 
	 << "::run: listen failed" << endl;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << CLASS_SIG << "::run: listen OK!" << endl;
#endif
    
  for (;;) {
    clientLen = sizeof(clientAddr);

#ifdef DEBUG
    cerr << CLASS_SIG << "::run: accept ready!" << endl;
#endif
    
    newsockfd = accept(sockfd, (SockAddr*)&clientAddr, &clientLen);
    if (0 > newsockfd) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::run: accept failed" << endl;
      //throw std::exception();
      // ONX̐݌vKv 
    }

#ifdef DEBUG
    cerr << CLASS_SIG << "::run: accept OK!" << endl;
#endif

    if (0 > write(newsockfd, getNewId())) {
      cerr << TAG_ERR << CLASS_SIG 
	   << "::run: write failed" << endl;
      //throw std::exception();
      // ONX̐݌vKv 
    }
      
#if defined(WIN32)
	closesocket(newsockfd);
#else
    close(newsockfd);
#endif
  }
}


std::string IdManager::getNewId()
{
  //_ł̃VXe𕶎YYYYMMDDHH24MMSS
  //\IDƂB
  const int BSIZE = 4096;
  char buf[BSIZE];
  time_t cur_time;
  time(&cur_time);
  struct tm* cur_tm = localtime(&cur_time);
  //size_t len = strftime(buf, BSIZE, "%Y%m%d%H%M%S", cur_tm);
  strftime(buf, BSIZE, "%Y%m%d%H%M%S", cur_tm);
  std::string mainId(buf);
  
  //r
  //ꎞłȂ΃V[PXԍZbg
  if (mainId != prevId_) {
    idSequence_ = 0;
  }
  //ێ
  prevId_ = mainId;
#if defined(WIN32)
  _snprintf(buf, BSIZE, "%s%05d", mainId.c_str(), ++idSequence_);
#else
  snprintf(buf, BSIZE, "%s%05d", mainId.c_str(), ++idSequence_);
#endif

#ifdef DEBUG
  cerr << "getNewId: ID=[" << buf << "]" << endl;
#endif
  
  //return DataRetainer::Id(buf);
  return buf;
}

#if 0
int 
main(int argc, char* argv[])
{
#if !defined(NO_LOGGER)
  // set log level  
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  Logger::printLog(NULL, " IdManager start", LOG_INFO,
                   0x00100003L, "exe");
#endif

#if !defined(WIN32)
  // make it to daemon & write pid file

  daemon(0, 1);

  pid_t pid = getpid();
  
  ofstream pidFile(IDMGR_PID_FILE.c_str());
  if (!pidFile) {
    cerr << "Error: pid file [" 
	 << IDMGR_PID_FILE << "] cannot open!" << endl;
    exit(1);
  }
  pidFile << pid;
  pidFile.close();
#endif

  // create instance of IdManager & run
  IdManager idMngr;
  idMngr.run();

#if !defined(NO_LOGGER)
  //write stop message
  Logger::printLog(NULL, " IdManager stop", LOG_INFO,
                   0x00100003L, "exe");
  Logger::term();
#endif

  return 0;
}
#endif

