/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Queue.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#if defined(WIN32)
#include <process.h>
#else
#include <unistd.h>
#endif

#include <algorithm>
#include <iostream>
#include "ServerCommon.h"
#include "ChannelDescriptor.h"
#include "ForwarderChnlSelector.h"
#include "SrvDrvChnlSelector.h"
#include "Queue.h"
#include "StringUtil.h"
#include "FileIDFunc.h"
#include "SrvConf.h"

using namespace OpenSOAP;
using namespace std;

#if defined(WIN32)
int Queue::mtCount_ = 0;
#endif

// RXgN^
// Fe[uobNAbvt@C̃pXO
//
Queue::Queue(std::string filePass, bool isFwdQueue)
{
#ifdef DEBUG
  cerr << "Queue::Queue Constructor" << endl;
#endif

  TableRecord tableRecord;
  isFwdQueue_ = isFwdQueue;
  
  queueTable.clear();

  SrvConf* srvConf = new SrvConf();
  tableFile = srvConf->getAsyncTablePath() + filePass;
  delete srvConf;

  // R[ht@Cǂݍ݁iȂ΍j
  tin.open(tableFile.c_str(), ios::in);
  if (tin.fail()) {
#ifdef DEBUG
    cerr << "Create table file" << endl;
#endif
  }
  while (!tin.eof() && !tin.fail()) {
    tin >> tableRecord.fileID;
    tableRecord.proccessFlag = INITIAL_STATE;
    if (!tin.eof()) {
#ifdef DEBUG
      cerr << "init: " << tableRecord.fileID << "\t" 
	   << tableRecord.proccessFlag << endl;
#endif
      queueTable.push_back(tableRecord);    
    }
  }
  tin.close();

  // MutexCϐCXbh̏
#if defined(WIN32)
	if (mtCount_ > 10000) {
		mtCount_ = 0;
	}
	std::string mtId = (isFwdQueue ? "FWDQUEUEMTX" : "QUEUEMTX") 
					+ StringUtil::toString(mtCount_++);
	queue_mutex = CreateMutex(NULL, FALSE, mtId.c_str());
	//queue_mutex = CreateMutex(NULL, FALSE, "QUEUEMTX");
	string semId = (isFwdQueue ? "FWDQUEUESEM" : "QUEUESEM") 
					+ StringUtil::toString(mtCount_);
	queue_semaphore = CreateSemaphore(NULL, 0, 99999, semId.c_str());
	//queue_semaphore = CreateSemaphore(NULL, 0, 99999, "QUEUESEM");
#else
  pthread_mutex_init(&queue_mutex, NULL);
  pthread_cond_init(&queue_cond, NULL);
#endif

  for (int thrd = 0; thrd < MAX_POP_THREAD_NUMBER ; thrd++) {
#if defined(WIN32)
	pop_thread[thrd] = (HANDLE)_beginthread(pop, 0, this);
#else
    pthread_create(&pop_thread[thrd], NULL, pop, this);
#endif
  }
}


// e[ut@C̏o֐
//
int Queue::fileOut()
{
  tout.open(tableFile.c_str(), ios::out);
  if (tout.fail()) {
    cerr << "Table_file output open error" << endl;
    return 1;
    exit(1);
  }

  for (unsigned int i=0; i<queueTable.size(); i++) {
    tout << queueTable[i].fileID << endl ;
    tout.flush();
#ifdef DEBUG
    cerr << queueTable[i].fileID << "\tFlag: "
	 << queueTable[i].proccessFlag << endl;
#endif
  }

  tout.close();
  return 0;
}

// Push\bh
// FiID
//
int Queue::push(string ID)
{
  // [J\̂̃R[hioFfileIDj
  TableRecord tableRecord;

  tableRecord.fileID = ID;
  tableRecord.proccessFlag = int(INITIAL_STATE);

  // mutex̃bN
#if defined(WIN32)
	WaitForSingleObject(queue_mutex, INFINITE);
#else
  int sl = pthread_mutex_lock(&(queue_mutex));
  if (sl == 0) {
#ifdef DEBUG
    cerr << "push thread mutex locked" << endl;
#endif
  }
#endif

  //  Queue̍Ōɒǉ
  queueTable.push_back(tableRecord);
  
#ifdef DEBUG
  cerr << "push" << endl;
  cerr << "vector size = " << queueTable.size() << endl ;
#endif

  // Queuee[ũt@Co
  int sf = fileOut();
  if (sf != 0) {
    cerr << "Table file write error" << endl;
    return 1;
  }

  // PopXbh֏ϐ̑M
#if defined(WIN32)
  // semaphore up(for windows)
  ReleaseSemaphore(queue_semaphore, 1, NULL);
#else
  // for pthread
  int sc = pthread_cond_signal(&(queue_cond));
  if (sc == 0) {
#ifdef DEBUG
    cerr << "push thread condition value sent" << endl;
#endif
  }
#endif

  // Mutex̉
#if defined(WIN32)
	ReleaseMutex(queue_mutex);
#else
  int su = pthread_mutex_unlock(&(queue_mutex));
  if (su == 0) {
#ifdef DEBUG
    cerr << "push thread mutex unlocked" << endl;
#endif
  }
#endif

	return 0;
}

// PopXbh
//
#if defined(WIN32)
void
#else
void* 
#endif
Queue::pop(void* arg)
{
  string ID;
  int status;

#if defined(WIN32)
	DWORD tid = GetCurrentThreadId();
#else
  pthread_t tid = pthread_self();
#endif

#ifdef DEBUG
  cerr << "thread ID: "<< tid << " created" << endl;
#endif

  // CX^Xւ̃|C^
  Queue* that = (Queue*)arg;
  if (NULL == that) {
    cerr << "Invalid argument" << endl;
  }

  // PopXbh̃C[v
  while (1) {

#if defined(WIN32)
	// semaphore down (for windows)
	WaitForSingleObject(that->queue_semaphore, INFINITE);
#endif

    // Mutex̃bN
#if defined(WIN32)
	WaitForSingleObject(that->queue_mutex, INFINITE);
#else
    status = pthread_mutex_lock(&(that->queue_mutex));
    if (status == 0) {
#ifdef DEBUG
      cerr << tid << " pop thread mutex (read) locked" << endl;
#endif
    }
#endif

#if !defined(WIN32)
    // Queue܂ĂȂꍇMutexC
    // Push\bh̏ϐ҂
    //vector<tableRecord>::iterator p = that->queueTable.end();
    int tableSize = that->queueTable.size();
    if (tableSize <= 0 || 
	that->queueTable[tableSize-1].proccessFlag != INITIAL_STATE) {
#ifdef DEBUG
      cerr << tid << " condition value waiting..." << endl;
#endif
      status = pthread_cond_wait(&(that->queue_cond), &(that->queue_mutex));
      if (status == 0) {
#ifdef DEBUG
		cerr << tid << " condition value received from puth method" << endl;
#endif
      }
    }
#endif

    // Queue̐擪R[hitÔȂ́j̓ǂݏo
    vector<TableRecord>::iterator itrRead = that->queueTable.begin();
    while (itrRead->proccessFlag != INITIAL_STATE
	    && itrRead != that->queueTable.end()) itrRead++;
    
    ID = itrRead->fileID;
#ifdef DEBUG
    cerr << "Poped ID: "<< ID << " Flag: " << itrRead->proccessFlag << endl;
#endif
    itrRead->proccessFlag = PROGRESS;
    that->fileOut();
    
    // Mutex̉
#if defined(WIN32)
	ReleaseMutex(that->queue_mutex);
#else
    status = pthread_mutex_unlock(&(that->queue_mutex));
    if (status == 0) {
#ifdef DEBUG
      cerr << tid << " pop thread mutex (read) unlocked" << endl;
#endif
    }
#endif

    // SrvDrv ւ̓]\bh
    status = that->popOutToNextDrv(ID);
    if (status == 0) {
#ifdef DEBUG
      cerr << tid << " Pop Output Successed" << endl;
#endif
    }
    
    // ㏈iQueueRecord̏Ct@Coj
    status = that->queueEraseAndFileOut(ID);
    if (status == 0) {
#ifdef DEBUG
      cerr << tid << " Record Erase Successed" << endl;
#endif
    }
  }

#if defined(WIN32)
	_endthread();
#else
	pthread_exit(NULL);
  return NULL;
#endif

}

// // SrvDrv ւ̓]\bh
//
bool
Queue::popOutToNextDrv(string fileID) {

#if defined(WIN32)
	DWORD tid = GetCurrentThreadId();
#else
  pthread_t tid = pthread_self();
#endif

  string nextDrv;
  if (isFwdQueue_) {
    nextDrv = "Fwd";
  }
  else {
    nextDrv = "SrvDrv";
  }

#ifdef DEBUG
  cerr << tid << " Async" << nextDrv << " thread wait" << endl;
#endif

  // Service Driver ̕ԓ҂
  
  ChannelDescriptor nextDrvChnlDesc;
  ChannelSelector* nextDrvChnlSelector;
  if (isFwdQueue_) {
    nextDrvChnlSelector = new ForwarderChnlSelector();
  }
  else {
    nextDrvChnlSelector = new SrvDrvChnlSelector(true);
  }
  if (0 != nextDrvChnlSelector->open(nextDrvChnlDesc)) {
    cerr << "Queue::popOutToNextDrv: " 
	 << nextDrv << "Chnl open failed" << endl;
    return false;
  }
  std::string fileIDOfMsgFromMsgDrv = fileID;
  if (0 > nextDrvChnlDesc.write(fileIDOfMsgFromMsgDrv)) {
    cerr << "Queue::popOutToNextDrv: write failed to " << nextDrv << endl;
    return false;
  }

#ifdef DEBUG
  cerr << "** Queue(1) ** WRITE To " << nextDrv << "=[" 
       << fileIDOfMsgFromMsgDrv << "]" << endl;
#endif

  std::string fileIDOfMsgFromNextDrv;
  if (0 > nextDrvChnlDesc.read(fileIDOfMsgFromNextDrv)) {
    cerr << "Queue::popOutToNextDrv: read failed from " << nextDrv << endl;
    return false;
  }

#ifdef DEBUG
  cerr << "** Queue ** READ From " << nextDrv << "=[" 
       << fileIDOfMsgFromNextDrv << "]" << endl;
#endif

  nextDrvChnlSelector->close(nextDrvChnlDesc);
  delete nextDrvChnlSelector;
  
#ifdef DEBUG
  cerr << tid << " Async" << nextDrv << " thread end" << endl;
#endif

  return true;
}


// ㏈iQueueRecord̏Ct@Coj
//
int Queue::queueEraseAndFileOut(string fileID) {

  int status;
#if defined(WIN32)
	DWORD tid = GetCurrentThreadId();
#else
  pthread_t tid = pthread_self();
#endif

  // Mutex̍ăbN
#if defined(WIN32)
	WaitForSingleObject(queue_mutex, INFINITE);
#else
  status = pthread_mutex_lock(&(queue_mutex));
  if (status == 0) {
#ifdef DEBUG
    cerr << tid << " pop thread mutex (rewrite) locked" << endl;
#endif
  }
#endif

  // Queuȅς݃R[h̏
  vector<TableRecord>::iterator itrErase = queueTable.begin();
  while (itrErase->fileID != fileID
	  && itrErase != queueTable.end()) itrErase++;
#ifdef DEBUG
  cerr << "File ID: " << itrErase->fileID << " erased" << endl;
#endif
  queueTable.erase(itrErase);
  
#ifdef DEBUG
  cerr << "vector size = " << queueTable.size() << endl ;
#endif

  // Queuee[ũt@Co
  status = fileOut();
  if (status != 0) {
    cerr << "Table file write error" << endl;
  }

  // Mutex̉
#if defined(WIN32)
	ReleaseMutex(queue_mutex);
#else
  status = pthread_mutex_unlock(&(queue_mutex));
  if (status == 0) {
#ifdef DEBUG
    cerr << tid << " pop thread mutex (rewrite) unlocked" << endl;
#endif
  }
#endif

	return 0;
}


// fXgN^
//
Queue::~Queue()
{
#ifdef DEBUG
  cerr << "Queue::~Queue Destructor" << endl;
#endif

  for (int thrd = 0; thrd < MAX_POP_THREAD_NUMBER ; thrd++) {
#if defined(WIN32)
	//CloseHandle(pop_thread[thrd]);
#else
    pthread_join(pop_thread[thrd], NULL);
#endif
  }

#if defined(WIN32)
	CloseHandle(queue_mutex);
	CloseHandle(queue_semaphore);
#else
	pthread_mutex_destroy(&queue_mutex);
	pthread_cond_destroy(&queue_cond);
#endif
}

void 
Queue::setChnlDesc(const ChannelDescriptor& chnlDesc)
{
#ifdef DEBUG
  cerr << "Queue::setChnlDesc" << endl;
#endif

  chnlDesc_ = chnlDesc;
}

bool 
Queue::execPushOpr()
{
  // MsgDrv 瑗Ăf[^ (t@C ID) ǂݍށB
  string fileIDOfMsgFromMsgDrv;
  if (0 > chnlDesc_.read(fileIDOfMsgFromMsgDrv)) {
    cerr << TAG_ERR << "Queue::execPushOpr: read failed" << endl;
    return false;
  }
  
#ifdef DEBUG
  cerr << "** Queue ** READ From MsgDrv=[" 
       << fileIDOfMsgFromMsgDrv << "]" << endl;
#endif

  // L[ɓB
  push(fileIDOfMsgFromMsgDrv);
  
  // L[ɓƂ̊mFbZ[W MsgDrv ɕԂB
  string msgToMsgDrv("Queue Push success");
  string fileIDOfMsgToMsgDrv 
    = convertToFileID(msgToMsgDrv, srvConf.getSoapMessageSpoolPath());
  if (0 > chnlDesc_.write(fileIDOfMsgToMsgDrv)) {
    cerr << TAG_ERR << "Queue::execPushOpr: read failed" << endl;
    return false;
  }
  
#ifdef DEBUG
  cerr << "** Queue ** WRITE To MsgDrv=[" 
       << fileIDOfMsgToMsgDrv << "]" << endl;
#endif

  return true;
}

