/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: TTL.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#ifndef TTL_H
#define TTL_H

#if defined(WIN32)
#include <windows.h>
#else
#include <unistd.h>
#include <pthread.h>
#endif

//#include <fstream.h>
#include <fstream>
#include <map>
#include <string>
#include <ctime>

#include "SrvConf.h"

namespace OpenSOAP {
  class ChannelDescriptor;
  
  class TTL {
  private:
    SrvConf srvConf;

#if defined(WIN32)
	HANDLE watcher_thread;
	HANDLE ttl_mutex;
	static int mtCount;
#else
    pthread_t watcher_thread;
    pthread_mutex_t ttl_mutex;
#endif

    std::string tableFile;
    std::ifstream min;
    std::ofstream mout;

    ChannelDescriptor chnlDesc_;  
    
  public:
    int watchInterval;
    int ttlMaxValue;
    
  public:
    TTL(std::string filePass);
    ~TTL();
    void setChnlDesc(const ChannelDescriptor& chnlDesc);
    
    bool execPushOpr();
    bool execRefOpr();
    

  private:
    int fileOut();
    static 
#if defined(WIN32)
	void
#else
	void* 
#endif
		watcher(void *arg);
    int eraseSignalToFIFO(std::string msgID, std::string fileID);
    
    int push(std::string msgID, std::string fileID, time_t startTime, 
	     int ttl, std::string backwardPath);
    std::string refBackwardPath(std::string msgID);
  private:
    struct mapRecord{
      std::string msgID;
      std::string fileID;
      time_t startTime;
      int ttl;
      std::string backwardPath;
    };
    
    std::multimap<time_t, mapRecord> ttlMap;
  };
}

#endif /* TTL_H */
