/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: soapInterface.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <string>
#include <vector>
#include <sys/types.h>

/* Channal Manager library */
#include "ChannelDescriptor.h"
#include "MsgDrvChnlSelector.h"

/* Common library */
#include "FileIDFunc.h"
#include "connection.h"
#include "SOAPMessage.h"
#include "StringUtil.h"
#include "SrvConf.h"

using namespace OpenSOAP;
using namespace std;

void
sendStdoutCGI(const std::string& soapMsg)
{
  int addcnt = 0;
#if defined(WIN32)
  const char* chkstr = soapMsg.c_str();
  for(; *chkstr != '\0'; ++chkstr) {
    if (*chkstr == '\n') {
      addcnt++;
    }
  }
#endif
  // Wo͂ɏoB
  // edit 2002/07/09 : EUC-JP -> UTF-8
  //sendStdout ("Content-Type: text/xml; charset=UTF-8\r\n");
  std::string headerMsg;
  headerMsg = "Content-Type: text/xml; charset=UTF-8\r\n";
  headerMsg += "Content-Length: ";
  headerMsg += StringUtil::toString(soapMsg.length()+addcnt);
  headerMsg += "\r\n";
  sendStdout (headerMsg);
  sendStdout (soapMsg);
}

int
main (int argc, char* argv[])
{
  int timeout = 5;

#ifdef DEBUG
  cerr << "TRANS IF: read ready." << endl;
#endif
  
  //==============================
  //ʐM(START)
  //==============================

  // W͂ SOAP bZ[Wǂݍ
  std::string soapMsgFromClient;
  int recvSize = recvStdinCGI(soapMsgFromClient);
  if (recvSize < 0) {
    cerr << "TRANS IF: recvStdinCGI failed" << endl;
    return EXIT_FAILURE;
  }
  else if (recvSize > OPENSOAP_MESSAGE_LIMIT_SIZE) {

#ifdef DEBUG
    cerr << "LIMIT OVER:[" << makeLimitSizeOverMessage() << "]" << endl;
#endif
    sendStdoutCGI(makeLimitSizeOverMessage());
    return EXIT_SUCCESS;
  }
#ifdef DEBUG
	cerr << "*** read stdin::[" << soapMsgFromClient << "]***" << endl;
#endif

  ChannelSelector* chnlSelector = new MsgDrvChnlSelector();
  //t@CfBXNv^
  ChannelDescriptor chnlDesc;
  
  if (0 != chnlSelector->open(chnlDesc)) {
    cerr << "TRANS_IF: open failed" << endl;
    return EXIT_FAILURE;
  }

	SrvConf srvConf;

#ifdef DEBUG
	cerr << "*** srvConf:getSoapMessageSpoolPath=[" 
       << srvConf.getSoapMessageSpoolPath() << "] ***" << endl;
#endif

  // t@C ID 擾
  std::string fileIDOfMsgFromClientMsg 
    = convertToFileID(soapMsgFromClient, srvConf.getSoapMessageSpoolPath());
  
  // 
  cerr << "TRANS IF: write to MsgDrv ready." << endl;

  if (0 > chnlDesc.write(fileIDOfMsgFromClientMsg)) {
    cerr << "TRANS IF: write to MsgDrv failed" << endl;
    return EXIT_FAILURE;
  }
  cerr << "TRANS IF: write to MsgDrv done." << endl;
  
  // ǂݍ
  std::string fileIDOfMsgFromMsgDrv;
  std::string readData;
  if (0 > chnlDesc.read(fileIDOfMsgFromMsgDrv)) {
    cerr << "TRANS IF: read from MsgDrv failed" << endl;
    return EXIT_FAILURE;
  }

#ifdef DEBUG
  cerr << "TRANS IF::READ=[" << fileIDOfMsgFromMsgDrv << "]" << endl;
#endif
  //==============================
  //ʐM(END)
  //==============================
  //N[Y
  chnlSelector->close(chnlDesc);
  delete chnlSelector;

  // t@C ID t@C̓eɕϊ
  std::string soapMsgFromMsgDrv;
  soapMsgFromMsgDrv 
    = revertFromFileID (fileIDOfMsgFromMsgDrv, 
			srvConf.getSoapMessageSpoolPath());
  sendStdoutCGI(soapMsgFromMsgDrv);

  return EXIT_SUCCESS;

}
